/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "libmcve_main.h"
#include "mcve.h"


// Standard includes ... all platforms
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>

//Win32 includes
#ifdef WIN32
#include <io.h>
#include <winsock2.h>
#include <sys/timeb.h>
#else // !Win32
// UNIX/POSIX includes
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <dirent.h>
#ifdef MCVE_PTHREAD
#include <pthread.h>
#endif // MCVE_PTHREAD

#endif //Win32

//SSL includes
#ifdef ENABLE_SSL
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#endif
#include "libmcve_misc.h"
#include "libmcve_locking.h"



void MCVE_init_locks(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  pthread_mutex_init(&conn->in_lock, NULL);
  pthread_mutex_init(&conn->out_lock, NULL);
  pthread_mutex_init(&conn->queue_lock, NULL);
#else
  conn->in_lock=0;
  conn->out_lock=0;
  conn->queue_lock=0;
#endif // MCVE_PTHREAD
#endif //MCVE_THREADSAFE
}

void MCVE_destroy_locks(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  pthread_mutex_destroy(&conn->in_lock);
  pthread_mutex_destroy(&conn->out_lock);
  pthread_mutex_destroy(&conn->queue_lock);
#else

#endif // MCVE_PTHREAD
#endif //MCVE_THREADSAFE
}

// Try to lock it....if already locked...wait until lock removed
void MCVE_lock(MCVE_CONN *myconn, int what)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  if (what == MCVE_LOCK_IN)
    pthread_mutex_lock(&conn->in_lock);
  else if (what == MCVE_LOCK_OUT)
    pthread_mutex_lock(&conn->out_lock);
  else if (what == MCVE_LOCK_QUEUE)
    pthread_mutex_lock(&conn->queue_lock);
#else
  int unlocked=0;
  while (!unlocked) {

    if (what == MCVE_LOCK_IN) {
      if (conn->in_lock) {
 //        printf("Waiting inlock\r\n");
         unlocked=0;
         MCVE_uwait(100);
      } else {
  //      printf("Locked INLOCK\r\n");
        conn->in_lock=1;
        unlocked=1;
      }
    } else if (what == MCVE_LOCK_OUT) {
      if (conn->out_lock) {
//         printf("Waiting outlock\r\n");
         unlocked=0;
         MCVE_uwait(100);
      } else {
 //       printf("Locked OUTLOCK\r\n");
        conn->out_lock=1;
        unlocked=1;
      }
    } else if (what == MCVE_LOCK_QUEUE) {
      if (conn->queue_lock) {
//        printf("Waiting authlock\r\n");
        unlocked=0;
        MCVE_uwait(100);
      } else {
  //      printf("Locked AUTHLOCK\r\n");
        conn->queue_lock=1;
        unlocked=1;
      }
    }

  }
#endif //MCVE_PTHREAD

#endif //MCVE_THREADSAFE
}

// Unlock it
void MCVE_unlock(MCVE_CONN *myconn, int what)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  if (what == MCVE_LOCK_IN)
    pthread_mutex_unlock(&conn->in_lock);
  else if (what == MCVE_LOCK_OUT)
    pthread_mutex_unlock(&conn->out_lock);
  else if (what == MCVE_LOCK_QUEUE)
    pthread_mutex_unlock(&conn->queue_lock);
#else
  if (what == MCVE_LOCK_IN)
    conn->in_lock=0;
  else if (what == MCVE_LOCK_OUT)
    conn->out_lock=0;
  else if (what == MCVE_LOCK_QUEUE)
    conn->queue_lock=0;
#endif // MCVE_PTHREAD
#endif // MCVE_THREADSAFE
}

