/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#ifndef __LIBMCVE_MAIN_H__
#define __LIBMCVE_MAIN_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef MCVE_PTHREAD
#include <pthread.h>
#endif

#ifndef HAVE_SOCKLEN_T
typedef int socklen_t;
#endif


#ifdef ENABLE_SSL
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#endif

#define MCVE_FILE 1
#define MCVE_SOCKETS 2
#define MCVE_SSL 3

/*
#define MCVE_SALE 1
#define MCVE_PREAUTH 2
#define MCVE_FORCE 3
#define MCVE_OVERRIDE 4
#define MCVE_RETURN 5
#define MCVE_VOID 6
#define MCVE_SETTLE 7
#define MCVE_CHKPWD 8
#define MCVE_ADDUSER 9
#define MCVE_DELUSER 10
#define MCVE_EDITUSER 11
#define MCVE_ENABLE 12
#define MCVE_DISABLE 13
#define MCVE_LISTUSERS 14
#define MCVE_GUT 15
#define MCVE_GL 16
#define MCVE_CHNGPWD 17
#define MCVE_UB 18
#define MCVE_GFT 19
#define MCVE_RS 20
#define MCVE_QC 21
#define MCVE_PBT 22
#define MCVE_BT 23
#define MCVE_LISTSTATS 24
#define MCVE_IMPORT 25
#define MCVE_EXPORT 26
*/


#define MCVE_LOCK_IN 1
#define MCVE_LOCK_OUT 2
#define MCVE_LOCK_QUEUE 3

#include "mcve.h"

/*
typedef struct MCVE_TRANS {
  int trans_type;
  int admin_type;
  char *username;
  char *password;
  char *account;
  char *trackdata;
  char *expdate;
  char *street;
  char *zip;
  char *cv;
  char *comments;
  char *clerkid;
  char *stationid;
  char *apprcode;
  double amount;
  long ptrannum;
  long sid;
  char *user;
  char *pwd;
  char *acct;
  char *bdate;
  char *edate;
  char *batch;
  char *file;
  int searchtype;
  int ping;
  MCVE_UserSetup  *usersetup;

} MCVE_TRANS;
*/

typedef struct MCVE_TRANS {
  char *key;
  char *value;
} MCVE_TRANS;


typedef struct MCVE_QUEUE {
  char identifier[31];
  int type;
  int admin;
  int status;

  /* Store outgoing transaction fields -- key/value pairs */
  int transaction_fields;
  MCVE_TRANS *transaction;


  // Stuff read from return
  int code;
  int avs;
  int cv;
  char *auth;
  char *text;
  MCVE_int64 tid;
  char *item;
  char *batch;
  
  int resp_fields;
  MCVE_TRANS *parsed_resp;
  // Comma Delimited response storage
  char *response;
  int iscommadelimiated;
  // storage
  char ***separated;
  int columns;
  int rows;
} MCVE_QUEUE;

#define MCVE_DISCONNECTED 0
#define MCVE_CONNECTED 1

typedef struct _MCVE_CONN {
  int conn_method;  // Drop File, Sockets, SSL
  char location[251]; // Either Drop-File directory, or hostname
  unsigned long port;  // For IP and SSL, a port number
  int ptr;  // For IP and SSL, the pointer to an open connection
  char *inbuf;
  char *outbuf;
  
  int status;

  long timeout;
  int blocking;
  int verifyconn;
  int verifyssl;
  int max_conn_time;
  char *error_text;
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  pthread_mutex_t in_lock;
  pthread_mutex_t out_lock;
  pthread_mutex_t queue_lock;
#else
  int in_lock;
  int out_lock;
  int queue_lock;
#endif // MCVE_PTHREAD
#endif // MCVE_THREADSAFE

#ifdef ENABLE_SSL
  SSL *ssl;
  X509 *server_cert;
#endif
  // int CA_verify;
  // char *CA_cert;  // To store an SSL certificate
  // char *CA_key;  // To store an SSL key
  long outstanding_auths;
  long queue_length;
  MCVE_QUEUE *queue;
} _MCVE_CONN;


#ifdef NO_ATOLL
#define atoll(a) atol(a)
#endif


#ifdef ENABLE_SSL
extern SSL_CTX *client_ctx;
#endif

#endif
