/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "libmcve_main.h"
#include "mcve.h"


// Standard includes ... all platforms
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>

//Win32 includes
#ifdef WIN32
#include <io.h>
#include <winsock2.h>
#include <sys/timeb.h>
#else // !Win32
// UNIX/POSIX includes
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <dirent.h>
#ifdef MCVE_PTHREAD
#include <pthread.h>
#endif // MCVE_PTHREAD

#endif //Win32

//SSL includes
#ifdef ENABLE_SSL
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#endif
#include "libmcve_misc.h"
#include "libmcve_file.h"
#include "libmcve_utils.h"
#include "libmcve_locking.h"

long MCVE_FileSize(FILE *fp)
{
  long pos, len;
  pos=ftell(fp);
  fseek(fp, 0L, SEEK_END);
  len=ftell(fp);
  fseek(fp, pos, SEEK_SET);
  return(len);
}

long MCVE_ReturnFiles(char ***filenames, char *directory)
{
  long files=0;

#ifdef WIN32
  struct _finddata_t c_file;
  long hFile;
  char find[250];

  filenames[0]=NULL;
  strcpy(find, directory);
  if (directory[strlen(find)-1] != '\\')
    strcat(find, "\\");
  strcat(find, "*.rtn");
  if ((hFile = _findfirst(find, &c_file)) == -1L) return(0);
  filenames[0]=(char **)realloc(filenames[0], (files+1)*sizeof(char));
  filenames[0][files]=(char *)malloc(251*sizeof(char));
  memset(filenames[0][files], 0, 251);
  strncpy(filenames[0][files], c_file.name, 250);
  files++;
  while (_findnext(hFile, &c_file) == 0) {
    filenames[0]=(char **)realloc(filenames[0], (files+1)*sizeof(char));
    filenames[0][files]=(char *)malloc(251*sizeof(char));
    memset(filenames[0][files], 0, 251);
    strncpy(filenames[0][files], c_file.name, 250);
    files++;
  }
  _findclose(hFile);
#else
  struct dirent *namelist;
  DIR *dir;
  char *ext;

  filenames[0]=NULL;
  dir=opendir(directory);
  if (dir == NULL) return(0);
  rewinddir(dir);
  while ((namelist=readdir(dir)) != NULL) {
    if (strlen(namelist->d_name) > 4)
       ext=namelist->d_name+(strlen(namelist->d_name)-4);
    else
      ext="";
    if (strcasecmp(ext, ".rtn") == 0) {
      filenames[0]=(char **)realloc(filenames[0], (files+1)*sizeof(char));
      filenames[0][files]=(char *)malloc(251*sizeof(char));
      memset(filenames[0][files], 0, 251);
      strncpy(filenames[0][files], namelist->d_name, 250);
      files++;
    }
  }
  closedir(dir);
#endif
  return(files);
}

int MCVE_Monitor_File(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long files, i;
  char **filenames=NULL;
  char *transaction;
  char *identifier;
  char filename[250];
  long flen;
  FILE *fp;

  files=MCVE_ReturnFiles(&filenames, conn->location);

  for (i=0; i<files; i++) {
    identifier=strdup(filenames[i]);
    identifier[strlen(identifier)-1]=0;
    strcpy(filename, conn->location);
    MCVE_DoCatSlash(filename);
    strcat(filename, filenames[i]);

    fp=fopen(filename, "rb");
    if (fp == NULL) {
       //can't open that file ... yet anyhow
       continue;
    }
    flen=MCVE_FileSize(fp);
    transaction=(char *)malloc(flen+2);
    memset(transaction, 0, flen+1);
    fread(transaction, flen,sizeof(char), fp);         // Read the contents to a buffer
    fclose(fp);

    MCVE_ProcessResponse(myconn, identifier, transaction);

    free(transaction);
    unlink(filename);  // Delete the file so it won't read it again
    free(filenames[i]);
  }

  if (files > 0)
    free(filenames);
  return(1);
}

int MCVE_SendTransaction_File(MCVE_CONN *myconn, char *identifier, char *transaction)
{
   _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
   char filename[300];
   FILE *fp;

   strcpy(filename, conn->location);
   if (filename[strlen(filename)-1] != '/')
     strcat(filename, "/");
   strcat(filename, identifier);
   strcat(filename, ".trn");
   fp=fopen(filename, "wb");
   if (fp == NULL) return(0);
   fwrite(transaction, strlen(transaction), 1, fp);
   fclose(fp);
   return(1);
}

