/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "libmcve_main.h"
#include "mcve.h"


// Standard includes ... all platforms
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>

//Win32 includes
#ifdef WIN32
#include <io.h>
#include <winsock2.h>
#include <sys/timeb.h>
#else // !Win32
// UNIX/POSIX includes
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <dirent.h>
#ifdef MCVE_PTHREAD
#include <pthread.h>
#endif // MCVE_PTHREAD

#endif //Win32

//SSL includes
#ifdef ENABLE_SSL
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#endif

#include "libmcve_utils.h"
#include "libmcve_misc.h"
#include "libmcve_parsing.h"
#include "libmcve_cdparsing.h"
#include "libmcve_file.h"
#include "libmcve_sockets.h"
#include "libmcve_locking.h"



long MCVE_FindQueueNumber(MCVE_CONN *myconn, char *identifier)
{
  long i;
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  for (i=0; i<conn->queue_length; i++) {
    if (conn->queue[i].status == MCVE_PENDING)
      if (strcmp(conn->queue[i].identifier, identifier) == 0)
         return(i);
  }
  return(-1);
}

void MCVE_StoreResponseValues(MCVE_CONN *myconn, long queuenum, char *option, char *value)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  if (strcmp(option, "code") == 0) {
    if (strcmp(value, "AUTH") == 0)
      conn->queue[queuenum].code=MCVE_AUTH;
    else if (strcmp(value, "DENY") == 0)
      conn->queue[queuenum].code=MCVE_DENY;
    else if (strcmp(value, "DUPL") == 0)
      conn->queue[queuenum].code=MCVE_DUPL;
    else if (strcmp(value, "CALL") == 0)
      conn->queue[queuenum].code=MCVE_CALL;
    else if (strcmp(value, "PKUP") == 0)
      conn->queue[queuenum].code=MCVE_PKUP;
    else if (strcmp(value, "RETRY") == 0)
      conn->queue[queuenum].code=MCVE_RETRY;
    else if (strcmp(value, "SETUP") == 0)
      conn->queue[queuenum].code=MCVE_SETUP;
    else if (strcmp(value, "SUCCESS") == 0)
      conn->queue[queuenum].code=MCVE_SUCCESS;
    else if (strcmp(value, "FAIL") == 0)
      conn->queue[queuenum].code=MCVE_FAIL;
    else if (strcmp(value, "TIMEOUT") == 0)
      conn->queue[queuenum].code=MCVE_TIMEOUT;
   else
      conn->queue[queuenum].code=MCVE_DENY;
  } else if (strcmp(option, "avs") == 0) {
    if (strcmp(value, "GOOD") == 0)
      conn->queue[queuenum].avs=MCVE_GOOD;
    else if (strcmp(value, "BAD") == 0)
      conn->queue[queuenum].avs=MCVE_BAD;
    else if (strcmp(value, "STREET") == 0)
      conn->queue[queuenum].avs=MCVE_STREET;
    else if (strcmp(value, "ZIP") == 0)
      conn->queue[queuenum].avs=MCVE_ZIP;
    else
      conn->queue[queuenum].avs=MCVE_UNKNOWN;
  } else if (strcmp(option, "cv") == 0) {
    if (strcmp(value, "GOOD") == 0)
      conn->queue[queuenum].cv=MCVE_GOOD;
    else if (strcmp(value, "BAD") == 0)
      conn->queue[queuenum].cv=MCVE_BAD;
    else
      conn->queue[queuenum].cv=MCVE_UNKNOWN;
  } else if (strcmp(option, "auth") == 0) {
    conn->queue[queuenum].auth=strdup(value);
  } else if (strcmp(option, "verbiage") == 0) {
    conn->queue[queuenum].text=strdup(value);
  } else if (strcmp(option, "item") == 0) {
    conn->queue[queuenum].item=strdup(value);
  } else if (strcmp(option, "batch") == 0) {
    conn->queue[queuenum].batch=strdup(value);
  } else if (strcmp(option, "tid") == 0 || strcmp(option, "sid") == 0) {
    conn->queue[queuenum].tid=atol(value);
  }
}

void MCVE_ProcessResponse(MCVE_CONN *myconn, char *identifier, char *message)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long num;
  long pos=0;
  char *line=NULL, *option=NULL, *value=NULL;
  int is_comma_delimiated=0;

  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  num=MCVE_FindQueueNumber(myconn, identifier);
  if (num == -1) {
    MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
    return;
  }
  conn->queue[num].status=MCVE_DONE;

  while ((pos=MCVE_ReadLine(message, pos, &line)) != -1) {
     if (!MCVE_ParseLine(line, &option, &value)) {
       is_comma_delimiated=1;
       conn->queue[num].iscommadelimiated=1;
       free(line);
       break;
     }
     MCVE_StoreResponseValues(myconn, num, option, value);
     conn->queue[num].parsed_resp=(MCVE_TRANS *)realloc(conn->queue[num].parsed_resp, (conn->queue[num].resp_fields+1)*sizeof(MCVE_TRANS));
     conn->queue[num].parsed_resp[conn->queue[num].resp_fields].key=MC_SAFE_strdup(option);
     conn->queue[num].parsed_resp[conn->queue[num].resp_fields].value=MC_SAFE_strdup(value);
     conn->queue[num].resp_fields++;
     free(option);
     free(value);
     free(line);
  }
  if (is_comma_delimiated) {
    conn->queue[num].response=(char *)malloc(strlen(message)+1);
    strcpy(conn->queue[num].response, message);
    conn->queue[num].code=MCVE_SUCCESS;
  }
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
}

void MCVE_ProcessBuffer(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int cutoff=0, done=0;
  char *temp=NULL, *temp2=NULL;
  char *response=NULL, *identifier=NULL, *message=NULL;

  MCVE_lock(myconn, MCVE_LOCK_IN);
  while (!done) {
    if (conn->inbuf != NULL) {
   //   printf("Checking for a response\r\n");
      cutoff=MCVE_IP_Get_Response(conn->inbuf, &response);
      if (cutoff != -1) {
        // Separate out the identifier and message
    //    printf("Got a full response!\r\n");
        if (MCVE_IP_Get_Response_Message(response, &message) && MCVE_IP_Get_Response_Identifier(response, &identifier)) {
           MCVE_ProcessResponse(myconn, identifier, message);
        }
        if (message != NULL) {  free(message); }
        if (identifier != NULL) {  free(identifier); }
        free(response);

        // --------------CUT TRANSACTION OUT OF INBUF ------------------
        temp=conn->inbuf+cutoff;
        if (strlen(temp) > 0) {
          temp2=strdup(temp);
          temp=NULL;
          conn->inbuf=(char *)realloc(conn->inbuf, strlen(temp2)+1);
          memset(conn->inbuf, 0, strlen(temp2)+1);
          strcpy(conn->inbuf, temp2);
        } else {
          free(conn->inbuf);
          conn->inbuf=NULL;
        }
        // -------------END CUTTING TRANSACTION OUT OF INBUF--------
      } else
        done=1;
    } else
      done=1;
  }
  MCVE_unlock(myconn, MCVE_LOCK_IN);
}





/*
void MCVE_ClearTransaction(MCVE_TRANS *trans)
{
  trans->trans_type=-1;
  trans->admin_type=-1;
  trans->username=NULL;
  trans->password=NULL;
  trans->account=NULL;
  trans->trackdata=NULL;
  trans->expdate=NULL;
  trans->street=NULL;
  trans->zip=NULL;
  trans->cv=NULL;
  trans->comments=NULL;
  trans->clerkid=NULL;
  trans->stationid=NULL;
  trans->apprcode=NULL;
  trans->bdate=NULL;
  trans->edate=NULL;
  trans->acct=NULL;
  trans->batch=NULL;
  trans->searchtype=0;
  trans->sid=-1;
  trans->ptrannum=-1;
  trans->amount=0;
  trans->user=NULL;
  trans->pwd=NULL;
  trans->usersetup=NULL;
  trans->file=NULL;
  trans->ping=0;

}

char *MCVE_TransType(MCVE_TRANS trans)
{
  if (trans.trans_type == MCVE_SALE)
       return("action=SALE");
  else if (trans.trans_type == MCVE_PREAUTH)
       return("action=PREAUTH");
  else if (trans.trans_type == MCVE_FORCE)
       return("action=FORCE");
  else if (trans.trans_type == MCVE_RETURN)
      return("action=RETURN");
  else if (trans.trans_type == MCVE_VOID)
      return("action=VOID");
  else if (trans.trans_type == MCVE_OVERRIDE)
      return("action=OVERRIDE");
  else if (trans.trans_type == MCVE_SETTLE)
      return("action=SETTLE");
  else if (trans.trans_type == MCVE_LISTUSERS)
      return("action=LISTUSERS");
  else if (trans.trans_type == MCVE_LISTSTATS)
      return("action=LISTSTATS");
  else if (trans.trans_type == MCVE_ADDUSER)
      return("action=ADDUSER");
  else if (trans.trans_type == MCVE_EDITUSER)
      return("action=EDITUSER");
  else if (trans.trans_type == MCVE_DELUSER)
      return("action=DELUSER");
  else if (trans.trans_type == MCVE_EDITUSER)
      return("action=EDITUSER");
  else if (trans.trans_type == MCVE_CHKPWD)
      return("action=CHKPWD");
  else if (trans.trans_type == MCVE_ENABLE)
      return("action=ENABLEUSER");
  else if (trans.trans_type == MCVE_DISABLE)
      return("action=DISABLEUSER");
  else if (trans.trans_type == MCVE_GUT)
      return("action=admin\nadmin=GUT");
  else if (trans.trans_type == MCVE_GL)
      return("action=admin\nadmin=GL");
  else if (trans.trans_type == MCVE_CHNGPWD)
      return("action=CHNGPWD\n");
  else if (trans.trans_type == MCVE_IMPORT)
      return("action=IMPORT\n");
  else if (trans.trans_type == MCVE_EXPORT)
      return("action=EXPORT\n");
  else if (trans.trans_type == MCVE_UB)
      return("action=admin\nadmin=UB");
  else if (trans.trans_type == MCVE_GFT)
      return("action=admin\nadmin=GFT");
  else if (trans.trans_type == MCVE_PBT)
      return("action=admin\nadmin=PBT");
  else if (trans.trans_type == MCVE_QC)
      return("action=admin\nadmin=QC");
  else if (trans.trans_type == MCVE_RS)
      return("action=admin\nadmin=RS");
  else if (trans.trans_type == MCVE_BT)
      return("action=admin\nadmin=BT");


  return("");
}

char *MCVE_StructureTransaction(MCVE_TRANS trans, MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  char *transaction=NULL;
  char temp[30];

  transaction=(char *)malloc(1025);
  memset(transaction, 0, 1024);

  if (trans.ping) {
    sprintf(transaction, "PING\r\n");
    return(transaction);
  }

  sprintf(transaction, "username=%s\r\npassword=%s\r\n%s\r\n",
                            trans.username,trans.password,MCVE_TransType(trans));
  if (conn->timeout > 0) {
    sprintf(temp, "timeout=%ld\r\n", conn->timeout);
    strcat(transaction, temp);
  }
  if (trans.file != NULL && strlen(trans.file)) {
    strcat(transaction, "file=");
    strcat(transaction, trans.file);
    strcat(transaction, "\r\n");
  }
  if (trans.user != NULL && strlen(trans.user)) {
    strcat(transaction, "user=");
    strncat(transaction, trans.user, 30);
    strcat(transaction, "\r\n");
  }
  if (trans.pwd != NULL && strlen(trans.pwd)) {
    strcat(transaction, "pwd=");
    strncat(transaction, trans.pwd, 30);
    strcat(transaction, "\r\n");
  }
  if (trans.account != NULL && strlen(trans.account)) {
    strcat(transaction, "account=");
    strncat(transaction, trans.account, 30);
    strcat(transaction, "\r\n");
  }
  if (trans.expdate != NULL && strlen(trans.expdate)) {
    strcat(transaction, "expdate=");
    strncat(transaction, trans.expdate, 4);
    strcat(transaction, "\r\n");
  }
  if (trans.trackdata != NULL && strlen(trans.trackdata)) {
    strcat(transaction, "trackdata=");
    strncat(transaction, trans.trackdata, 199);
    strcat(transaction, "\r\n");
  }
  if (trans.amount > 0) {
    sprintf(temp, "amount=%.02f\r\n", trans.amount);
    strcat(transaction, temp);
  }
  if (trans.street != NULL && strlen(trans.street)) {
    strcat(transaction, "street=");
    strncat(transaction, trans.street, 49);
    strcat(transaction, "\r\n");
  }
  if (trans.zip != NULL && strlen(trans.zip)) {
    strcat(transaction, "zip=");
    strncat(transaction, trans.zip, 10);
    strcat(transaction, "\r\n");
  }
  if (trans.cv != NULL && strlen(trans.cv)) {
    strcat(transaction, "cvv2=");
    strncat(transaction, trans.cv, 5);
    strcat(transaction, "\r\n");
  }
  if (trans.comments != NULL && strlen(trans.comments)) {
    strcat(transaction, "comments=");
    strncat(transaction, trans.comments, 100);
    strcat(transaction, "\r\n");
  }
  if (trans.stationid != NULL && strlen(trans.stationid)) {
    strcat(transaction, "stationid=");
    strncat(transaction, trans.stationid, 30);
    strcat(transaction, "\r\n");
  }
  if (trans.clerkid != NULL && strlen(trans.clerkid)) {
    strcat(transaction, "clerkid=");
    strncat(transaction, trans.clerkid, 30);
    strcat(transaction, "\r\n");
  }
  if (trans.apprcode != NULL && strlen(trans.apprcode)) {
    strcat(transaction, "apprcode=");
    strncat(transaction, trans.apprcode, 10);
    strcat(transaction, "\r\n");
  }
  if (trans.ptrannum > 0) {
    sprintf(temp, "ptrannum=%ld\r\n", trans.ptrannum);
    strcat(transaction, temp);
  }
  if (trans.sid != -1) {
    sprintf(temp, "sid=%ld\r\n", trans.sid);
    strcat(transaction, temp);
  }
  if (trans.searchtype > 0) {
    strcat(transaction, "type=");
    if (trans.searchtype == MCVE_SALE)
      strcat(transaction, "SALE");
    else if (trans.searchtype == MCVE_PREAUTH)
      strcat(transaction, "PREAUTH");
    else if (trans.searchtype == MCVE_OVERRIDE)
      strcat(transaction, "OVERRIDE");
    else if (trans.searchtype == MCVE_RETURN)
      strcat(transaction, "RETURN");
    else if (trans.searchtype == MCVE_SETTLE)
      strcat(transaction, "SETTLE");
    else if (trans.searchtype == MCVE_FORCE)
      strcat(transaction, "FORCE");
    strcat(transaction, "\r\n");
  }
  if (trans.bdate != NULL && strlen(trans.bdate)) {
    strcat(transaction, "bdate=");
    strncat(transaction, trans.bdate, 10);
    strcat(transaction, "\r\n");
  }
  if (trans.edate != NULL && strlen(trans.edate)) {
    strcat(transaction, "edate=");
    strncat(transaction, trans.edate, 10);
    strcat(transaction, "\r\n");
  }
  if (trans.acct != NULL && strlen(trans.acct)) {
    strcat(transaction, "acct=");
    strncat(transaction, trans.acct, 25);
    strcat(transaction, "\r\n");
  }
  if (trans.batch != NULL && strlen(trans.batch)) {
    strcat(transaction, "batch=");
    strncat(transaction, trans.batch, 6);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup != NULL) {
  // Administrative stuff
  if (trans.usersetup->proc != NULL && strlen(trans.usersetup->proc)) {
    strcat(transaction, "proc=");
    strncat(transaction, trans.usersetup->proc, 12);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->user != NULL && strlen(trans.usersetup->user)) {
    strcat(transaction, "user=");
    strncat(transaction, trans.usersetup->user, 20);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->pwd != NULL && strlen(trans.usersetup->pwd)) {
    strcat(transaction, "pwd=");
    strncat(transaction, trans.usersetup->pwd, 12);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->indcode != NULL && strlen(trans.usersetup->indcode)) {
    strcat(transaction, "indcode=");
    strncat(transaction, trans.usersetup->indcode, 1);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->merchid != NULL && strlen(trans.usersetup->merchid)) {
    strcat(transaction, "merchid=");
    strncat(transaction, trans.usersetup->merchid, 25);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->bankid != NULL && strlen(trans.usersetup->bankid)) {
    strcat(transaction, "bankid=");
    strncat(transaction, trans.usersetup->bankid, 25);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->termid != NULL && strlen(trans.usersetup->termid)) {
    strcat(transaction, "termid=");
    strncat(transaction, trans.usersetup->termid, 12);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->clientnum != NULL && strlen(trans.usersetup->clientnum)) {
    strcat(transaction, "clientnum=");
    strncat(transaction, trans.usersetup->clientnum, 20);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->storeid != NULL && strlen(trans.usersetup->storeid)) {
    strcat(transaction, "storeid=");
    strncat(transaction, trans.usersetup->storeid, 20);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->agentid != NULL && strlen(trans.usersetup->agentid)) {
    strcat(transaction, "agentid=");
    strncat(transaction, trans.usersetup->agentid, 20);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->chainid != NULL && strlen(trans.usersetup->chainid)) {
    strcat(transaction, "chainid=");
    strncat(transaction, trans.usersetup->chainid, 20);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->zipcode != NULL && strlen(trans.usersetup->zipcode)) {
    strcat(transaction, "zipcode=");
    strncat(transaction, trans.usersetup->zipcode, 10);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->timezone != NULL && strlen(trans.usersetup->timezone)) {
    strcat(transaction, "timezone=");
    strncat(transaction, trans.usersetup->timezone, 2);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->merchcat != NULL && strlen(trans.usersetup->merchcat)) {
    strcat(transaction, "merchcat=");
    strncat(transaction, trans.usersetup->merchcat, 12);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->mername != NULL && strlen(trans.usersetup->mername)) {
    strcat(transaction, "mername=");
    strncat(transaction, trans.usersetup->mername, 26);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->merchloc != NULL && strlen(trans.usersetup->merchloc)) {
    strcat(transaction, "merchloc=");
    strncat(transaction, trans.usersetup->merchloc, 20);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->statecode != NULL && strlen(trans.usersetup->statecode)) {
    strcat(transaction, "statecode=");
    strncat(transaction, trans.usersetup->statecode, 3);
    strcat(transaction, "\r\n");
  }
  if (trans.usersetup->servicephone != NULL && strlen(trans.usersetup->servicephone)) {
    strcat(transaction, "servicephone=");
    strncat(transaction, trans.usersetup->servicephone, 20);
    strcat(transaction, "\r\n");
  }
  }
 // printf("Transaction:\r\n%s\r\n", transaction);
  return(transaction);

}

*/

long MCVE_OpenQueueSlot(MCVE_CONN *myconn)
{
   _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
   long i;
   long pos=-1;

   for (i=0; i<conn->queue_length; i++) {
     if (conn->queue[i].status == MCVE_UNUSED) {
        pos=i;
        break;
     }
   }
   if (pos == -1) {
     if (conn->queue == NULL) {
        conn->queue = (MCVE_QUEUE *)malloc(1 * sizeof(MCVE_QUEUE));
        pos=0;
     } else {
       conn->queue = (MCVE_QUEUE *)realloc(conn->queue, (conn->queue_length+1)*sizeof(MCVE_QUEUE));
       pos=conn->queue_length;
     }
     memset(conn->queue[pos].identifier, 0, 30);
     conn->queue[pos].status = MCVE_UNUSED;
     conn->queue_length++;
   }
   return(pos);
}



char *MCVE_StructureTransaction(MCVE_CONN *myconn, long identifier)
{
  char *transaction=NULL;
  char temp[512];
  int i;
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  transaction=(char *)malloc(1025);
  memset(transaction, 0, 1024);

  if (conn->queue[identifier].type == MC_TRAN_PING) {
    strcpy(transaction, "PING\r\n");
    return(transaction);
  }
  for (i=0; i<conn->queue[identifier].transaction_fields; i++) {
    sprintf(temp, "%s=%s\r\n", conn->queue[identifier].transaction[i].key, conn->queue[identifier].transaction[i].value);
    strcat(transaction, temp);
  }
  return(transaction);
}

int MCVE_ClearQueueSlot(MCVE_CONN *myconn, long pos)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  conn->queue[pos].type=0;
  conn->queue[pos].admin=0;
  conn->queue[pos].auth=NULL;
  conn->queue[pos].text=NULL;
  conn->queue[pos].item=NULL;
  conn->queue[pos].batch=NULL;
  conn->queue[pos].code=-1;
  conn->queue[pos].tid=-1;
  conn->queue[pos].resp_fields=0;
  conn->queue[pos].parsed_resp=NULL;
  conn->queue[pos].response=NULL;
  conn->queue[pos].separated=NULL;
  conn->queue[pos].transaction_fields=0;
  conn->queue[pos].transaction=NULL;
  return(1);
}

/*
long MCVE_AddOutstandingAuth(MCVE_CONN *myconn, char *identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long pos;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  pos=MCVE_OpenQueueSlot(myconn);
  strncpy(conn->queue[pos].identifier, identifier, 29);
  conn->queue[pos].status=MCVE_PENDING;
  conn->queue[pos].auth=NULL;
  conn->queue[pos].text=NULL;
  conn->queue[pos].item=NULL;
  conn->queue[pos].batch=NULL;
  conn->queue[pos].code=-1;
  conn->queue[pos].tid=-1;
  conn->queue[pos].response=NULL;
  conn->queue[pos].separated=NULL;
  conn->outstanding_auths++;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(pos);
}
*/

int MCVE_SendTransaction(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  char *transaction=NULL;
  char *id=NULL;
  int ret=0;

  id=MCVE_GenerateIdentifier();
  transaction=MCVE_StructureTransaction(myconn, identifier);

  if (conn->conn_method == MCVE_FILE)
    ret=MCVE_SendTransaction_File(myconn, id, transaction);
  else
    ret=MCVE_SendTransaction_IP(myconn, id, transaction);

  free(transaction);

  //pos=MCVE_AddOutstandingAuth(myconn, identifier);
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  conn->queue[identifier].status=MCVE_PENDING;
  strcpy(conn->queue[identifier].identifier, id);
  free(id);
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);

  return(ret);
}
